@@ -44,6 +44,9 @@
             current_table/2,            % :Variant, ?Table
             abolish_all_tables/0,
             abolish_table_subgoals/1,   % :Subgoal
+            abolish_module_tables/1,    % +Module
+            abolish_nonincremental_tables/0,
+            abolish_nonincremental_tables/1, % +Options
 
             start_tabling/2,            % +Wrapper, :Worker
             start_subsumptive_tabling/2,% +Wrapper, :Worker
@@ -51,7 +54,10 @@
 
             '$wrap_tabled'/2,		% :Head, +Mode
             '$moded_wrap_tabled'/4,	% :Head, +ModeTest, +Variant, +Moded
-            '$wfs_call'/2               % :Goal, -Delays
+            '$wfs_call'/2,              % :Goal, -Delays
+
+            '$wrap_incremental'/1,      % :Head
+            '$unwrap_incremental'/1     % :Head
           ]).
 
 :- meta_predicate
@@ -151,6 +157,9 @@ dyn_tabling_list([H|T], M) :-
     dyn_tabling(H, M),
     dyn_tabling_list(T, M).
 
+dyn_tabling(M:Clause, _) :-
+    !,
+    dyn_tabling(Clause, M).
 dyn_tabling((:- multifile(PI)), M) :-
     !,
     multifile(M:PI),
@@ -177,6 +186,15 @@ dyn_tabling('$table_mode'(Head, Variant, Moded), M) :-
         )
     ;   assertz(M:'$table_mode'(Head, Variant, Moded))
     ).
+dyn_tabling(('$table_update'(Head, S0, S1, S2) :- Body), M) :-
+    (   clause(M:'$table_update'(Head, S00, S10, S20), Body0, Ref)
+    ->  (   t(Head, S0, S1, S2, Body) =@= t(Head, S00, S10, S20, Body0)
+        ->  true
+        ;   erase(Ref),
+            assertz(M:('$table_update'(Head, S0, S1, S2) :- Body))
+        )
+    ;   assertz(M:('$table_update'(Head, S0, S1, S2) :- Body))
+    ).
 
 %!  untable(M:PIList) is det.
 %
@@ -194,6 +212,10 @@ untable(Var, _) :-
     var(Var),
     !,
     '$instantiation_error'(Var).
+untable(M:Spec, _) :-
+    !,
+    '$must_be'(atom, M),
+    untable(Spec, M).
 untable((A,B), M) :-
     !,
     untable(A, M),
@@ -235,32 +257,51 @@ untable(TableSpec, _) :-
 %   @compat This interface may change or disappear without notice
 %           from future versions.
 
-'$wrap_tabled'(Head, variant) :-
+'$wrap_tabled'(Head, Options) :-
+    get_dict(mode, Options, subsumptive),
     !,
-    '$set_predicate_attribute'(Head, tabled, true),
-    '$wrap_predicate'(Head, table, Wrapped,
-                      start_tabling(Head, Wrapped)).
-'$wrap_tabled'(Head, subsumptive) :-
-    !,
-    '$set_predicate_attribute'(Head, tabled, true),
+    set_pattributes(Head, Options),
     '$wrap_predicate'(Head, table, Wrapped,
                       start_subsumptive_tabling(Head, Wrapped)).
+'$wrap_tabled'(Head, Options) :-
+    !,
+    set_pattributes(Head, Options),
+    '$wrap_predicate'(Head, table, Wrapped,
+                      start_tabling(Head, Wrapped)).
+
+set_pattributes(Head, Options) :-
+    '$set_predicate_attribute'(Head, tabled, true),
+    (   get_dict(incremental, Options, true)
+    ->  '$set_predicate_attribute'(Head, incremental, true)
+    ;   true
+    ),
+    (   get_dict(dynamic, Options, true)
+    ->  '$set_predicate_attribute'(Head, dynamic, true)
+    ;   true
+    ).
 
 start_tabling(Wrapper, Worker) :-
     '$tbl_variant_table'(Wrapper, Trie, Status, Skeleton),
     (   Status == complete
-    ->  '$tbl_answer_update_dl'(Trie, Skeleton)
+    ->  '$idg_add_edge'(Trie),
+        '$tbl_answer_update_dl'(Trie, Skeleton)
     ;   Status == fresh
     ->  '$tbl_create_subcomponent'(SCC, Trie),
         tdebug(user_goal(Wrapper, Goal)),
         tdebug(schedule, 'Created component ~d for ~p', [SCC, Goal]),
+        '$idg_add_edge'(OldCurrent, Trie),
         setup_call_catcher_cleanup(
             true,
             run_leader(Skeleton, Worker, Trie, SCC, LStatus),
             Catcher,
             finished_leader(Catcher, SCC, Wrapper)),
         tdebug(schedule, 'Leader ~p done, status = ~p', [Goal, LStatus]),
+        '$idg_set_current'(OldCurrent),
         done_leader(LStatus, SCC, Skeleton, Trie)
+    ;   Status == invalid
+    ->  reeval(Trie),
+        '$idg_add_edge'(Trie),
+        '$tbl_answer_update_dl'(Trie, Skeleton)
     ;   % = run_follower, but never fresh and Status is a worklist
         shift(call_info(Skeleton, Status))
     ).
@@ -271,22 +312,30 @@ start_tabling(Wrapper, Worker) :-
 start_subsumptive_tabling(Wrapper, Worker) :-
     (   '$tbl_existing_variant_table'(Wrapper, Trie, Status, Skeleton)
     ->  (   Status == complete
-        ->  '$tbl_answer_update_dl'(Trie, Skeleton)
+        ->  '$idg_add_edge'(Trie),
+            '$tbl_answer_update_dl'(Trie, Skeleton)
+        ;   Status == invalid
+        ->  reeval(Trie),
+            '$idg_add_edge'(Trie),
+            '$tbl_answer_update_dl'(Trie, Skeleton)
         ;    shift(call_info(Skeleton, Status))
         )
     ;   more_general_table(Wrapper, ATrie),
         '$tbl_table_status'(ATrie, complete, Wrapper, Skeleton)
-    ->  '$tbl_answer_update_dl'(ATrie, Skeleton)
+    ->  '$idg_add_edge'(ATrie),
+        '$tbl_answer_update_dl'(ATrie, Skeleton)
     ;   '$tbl_variant_table'(Wrapper, Trie, _0Status, Skeleton),
         tdebug(_0Status == fresh),
         '$tbl_create_subcomponent'(SCC, Trie),
         tdebug(user_goal(Wrapper, Goal)),
         tdebug(schedule, 'Created component ~d for ~p', [SCC, Goal]),
+        '$idg_add_edge'(OldCurrent, Trie),
         setup_call_catcher_cleanup(
             true,
             run_leader(Skeleton, Worker, Trie, SCC, LStatus),
             Catcher,
             finished_leader(Catcher, SCC, Wrapper)),
+        '$idg_set_current'(OldCurrent),
         tdebug(schedule, 'Leader ~p done, status = ~p', [Goal, LStatus]),
         done_leader(LStatus, SCC, Skeleton, Trie)
     ).
@@ -346,7 +395,7 @@ activate(Wrapper, Worker, Trie, WorkList) :-
     tdebug(activate, '~p: created wl=~p, trie=~p',
            [Wrapper, WorkList, Trie]),
     (   reset_delays,
-        delim(Wrapper, Worker, WorkList, []),   % FIXME: is this right?
+        delim(Wrapper, Worker, WorkList, []),
         fail
     ;   true
     ).
@@ -392,18 +441,25 @@ delim(Wrapper, Worker, WorkList, Delays) :-
 start_tabling(Wrapper, Worker, WrapperNoModes, ModeArgs) :-
     '$tbl_variant_table'(WrapperNoModes, Trie, Status, _Skeleton),
     (   Status == complete
-    ->  trie_gen(Trie, WrapperNoModes, ModeArgs)
+    ->  '$idg_add_edge'(Trie),
+        trie_gen(Trie, WrapperNoModes, ModeArgs)
     ;   Status == fresh
     ->  '$tbl_create_subcomponent'(SubComponent, Trie),
+        '$idg_add_edge'(OldCurrent, Trie),
         setup_call_catcher_cleanup(
             true,
             run_leader(Wrapper, WrapperNoModes, ModeArgs,
                        Worker, Trie, SubComponent, LStatus),
             Catcher,
             finished_leader(Catcher, SubComponent, Wrapper)),
+        '$idg_set_current'(OldCurrent),
         tdebug(schedule, 'Leader ~p done, modeargs = ~p, status = ~p',
                [Wrapper, ModeArgs, LStatus]),
         moded_done_leader(LStatus, SubComponent, WrapperNoModes, ModeArgs, Trie)
+    ;   Status == invalid
+    ->  reeval(Trie),
+        '$idg_add_edge'(Trie),
+        trie_gen(Trie, WrapperNoModes, ModeArgs)
     ;   % = run_follower, but never fresh and Status is a worklist
         shift(call_info(Wrapper, Status))
     ).
@@ -523,6 +579,7 @@ completion_step(WorkList) :-
         '$tbl_wkl_work'(WorkList,
                         Answer, ModeArgs,
                         Goal, Continuation, Wrapper, TargetWorklist, Delays),
+        '$idg_set_current_wl'(TargetWorklist),
         tdebug(wl_goal(WorkList, SourceGoal, _)),
         tdebug(wl_goal(TargetWorklist, TargetGoal, _Skeleton)),
         (   ModeArgs == Reserved
@@ -553,6 +610,7 @@ completion_step(WorkList) :-
 tnot(Goal0) :-
     '$tbl_implementation'(Goal0, Goal),         % verifies Goal is tabled
     '$tbl_variant_table'(Goal, Trie, Status, Skeleton),
+    \+ \+ '$idg_add_edge'(Trie),                % do not update current node
     (   '$tbl_answer_dl'(Trie, _, true)
     ->  fail
     ;   '$tbl_answer_dl'(Trie, _, _)
@@ -616,7 +674,10 @@ reset_delays :-
     reset_delays,
     call(Goal),
     '$tbl_delay_list'(DL1),
-    delay_goals(DL1, M, Delays),
+    (   delay_goals(DL1, M, Delays)
+    ->  true
+    ;   Delays = undefined
+    ),
     '$append'(DL0, DL1, DL),
     '$tbl_set_delay_list'(DL).
 
@@ -645,10 +706,13 @@ delay_goals([AT|T], M, Goal) :-
     ).
 
 at_delay_goal(tnot(Trie), M, tnot(Goal), Skeleton) :-
+    is_trie(Trie),
     !,
     '$tbl_table_status'(Trie, _Status, Wrapper, Skeleton),
     unqualify_goal(Wrapper, M, Goal).
 at_delay_goal(Trie, M, Goal, Skeleton) :-
+    is_trie(Trie),
+    !,
     '$tbl_table_status'(Trie, _Status, Wrapper, Skeleton),
     unqualify_goal(Wrapper, M, Goal).
 
@@ -678,7 +742,8 @@ abolish_all_tables :-
 %
 %   Abolish all tables that unify with SubGoal.
 
-abolish_table_subgoals(M:SubGoal) :-
+abolish_table_subgoals(SubGoal0) :-
+    '$tbl_implementation'(SubGoal0, M:SubGoal),
     '$tbl_variant_table'(VariantTrie),
     !,
     current_module(M),
@@ -686,6 +751,61 @@ abolish_table_subgoals(M:SubGoal) :-
            '$tbl_destroy_table'(Trie)).
 abolish_table_subgoals(_).
 
+%!  abolish_module_tables(+Module) is det.
+%
+%   Abolish all tables for predicates associated with the given module.
+
+abolish_module_tables(Module) :-
+    '$must_be'(atom, Module),
+    '$tbl_variant_table'(VariantTrie),
+    current_module(Module),
+    !,
+    forall(trie_gen(VariantTrie, Module:_, Trie),
+           '$tbl_destroy_table'(Trie)).
+abolish_module_tables(_).
+
+%!  abolish_nonincremental_tables is det.
+%
+%   Abolish all tables that are not related to incremental predicates.
+
+abolish_nonincremental_tables :-
+    '$tbl_variant_table'(VariantTrie),
+    (   trie_gen(VariantTrie, _, Trie),
+        '$tbl_table_status'(Trie, Status, Goal, _),
+        (   Status == complete
+        ->  true
+        ;   '$permission_error'(abolish, incomplete_table, Trie)
+        ),
+        \+ predicate_property(Goal, incremental),
+        '$tbl_destroy_table'(Trie),
+        fail
+    ;   true
+    ).
+
+%!  abolish_nonincremental_tables(+Options)
+%
+%   Allow for skipping incomplete tables while abolishing.
+%
+%   @tbd Mark tables for destruction such   that they are abolished when
+%   completed.
+
+abolish_nonincremental_tables(Options) :-
+    (   Options = on_incomplete(Action)
+    ->  Action == skip
+    ;   '$option'(on_incomplete(skip), Options)
+    ),
+    !,
+    '$tbl_variant_table'(VariantTrie),
+    (   trie_gen(VariantTrie, _, Trie),
+        '$tbl_table_status'(Trie, complete, Goal, _),
+        \+ predicate_property(Goal, incremental),
+        '$tbl_destroy_table'(Trie),
+        fail
+    ;   true
+    ).
+abolish_nonincremental_tables(_) :-
+    abolish_nonincremental_tables.
+
 
                  /*******************************
                  *        EXAMINE TABLES        *
@@ -713,41 +833,46 @@ current_table(M:Variant, Trie) :-
 :- dynamic
     system:term_expansion/2.
 
-wrappers(Spec) -->
-    wrappers(Spec, #{}).
+wrappers(Spec, M) -->
+    wrappers(Spec, M, #{}).
 
-wrappers(Var, _) -->
+wrappers(Var, _, _) -->
     { var(Var),
       !,
       '$instantiation_error'(Var)
     }.
-wrappers(Spec as Options, Opts0) -->
+wrappers(M:Spec, _, Opts) -->
+    !,
+    { '$must_be'(atom, M) },
+    wrappers(Spec, M, Opts).
+wrappers(Spec as Options, M, Opts0) -->
     !,
     { table_options(Options, Opts0, Opts) },
-    wrappers(Spec, Opts).
-wrappers((A,B), Opts) -->
+    wrappers(Spec, M, Opts).
+wrappers((A,B), M, Opts) -->
     !,
-    wrappers(A, Opts),
-    wrappers(B, Opts).
-wrappers(Name//Arity, Opts) -->
+    wrappers(A, M, Opts),
+    wrappers(B, M, Opts).
+wrappers(Name//Arity, M, Opts) -->
     { atom(Name), integer(Arity), Arity >= 0,
       !,
       Arity1 is Arity+2
     },
-    wrappers(Name/Arity1, Opts).
-wrappers(Name/Arity, Opts) -->
+    wrappers(Name/Arity1, M, Opts).
+wrappers(Name/Arity, Module, Opts) -->
     { '$option'(mode(TMode), Opts, variant),
       atom(Name), integer(Arity), Arity >= 0,
       !,
       functor(Head, Name, Arity),
-      prolog_load_context(module, Module),
       '$tbl_trienode'(Reserved)
     },
-    [ '$tabled'(Head, TMode),
-      '$table_mode'(Head, Head, Reserved),
-      (:- initialization('$wrap_tabled'(Module:Head, TMode), now))
+    qualify(Module,
+            [ '$tabled'(Head, TMode),
+              '$table_mode'(Head, Head, Reserved)
+            ]),
+    [ (:- initialization('$wrap_tabled'(Module:Head, Opts), now))
     ].
-wrappers(ModeDirectedSpec, Opts) -->
+wrappers(ModeDirectedSpec, Module, Opts) -->
     { '$option'(mode(TMode), Opts, variant),
       callable(ModeDirectedSpec),
       !,
@@ -755,23 +880,38 @@ wrappers(ModeDirectedSpec, Opts) -->
       functor(Head, Name, Arity),
       extract_modes(ModeDirectedSpec, Head, Variant, Modes, Moded),
       updater_clauses(Modes, Head, UpdateClauses),
-      prolog_load_context(module, Module),
       mode_check(Moded, ModeTest),
       (   ModeTest == true
-      ->  WrapClause = '$wrap_tabled'(Module:Head, TMode)
+      ->  WrapClause = '$wrap_tabled'(Module:Head, Opts)
       ;   WrapClause = '$moded_wrap_tabled'(Module:Head, ModeTest,
           Module:Variant, Moded)
       )
     },
-    [ '$tabled'(Head, TMode),
-      '$table_mode'(Head, Variant, Moded),
-      (:- initialization(WrapClause, now))
-    | UpdateClauses
-    ].
-wrappers(TableSpec, _Opts) -->
+    qualify(Module,
+            [ '$tabled'(Head, TMode),
+              '$table_mode'(Head, Variant, Moded)
+            ]),
+    [ (:- initialization(WrapClause, now))
+    ],
+    qualify(Module, UpdateClauses).
+wrappers(TableSpec, _M, _Opts) -->
     { '$type_error'(table_desclaration, TableSpec)
     }.
 
+qualify(Module, List) -->
+    { prolog_load_context(module, Module) },
+    !,
+    clist(List).
+qualify(Module, List) -->
+    qlist(List, Module).
+
+clist([])    --> [].
+clist([H|T]) --> [H], clist(T).
+
+qlist([], _)    --> [].
+qlist([H|T], M) --> [M:H], qlist(T, M).
+
+
 %!  table_options(+Options, +OptDictIn, -OptDictOut)
 %
 %   Handler the ... as _options_ ... construct.
@@ -780,13 +920,23 @@ table_options(Options, _Opts0, _Opts) :-
     var(Options),
     '$instantiation_error'(Options).
 table_options((A,B), Opts0, Opts) :-
+    !,
     table_options(A, Opts0, Opts1),
     table_options(B, Opts1, Opts).
 table_options(subsumptive, Opts0, Opts1) :-
+    !,
     put_dict(mode, Opts0, subsumptive, Opts1).
 table_options(variant, Opts0, Opts1) :-
+    !,
     put_dict(mode, Opts0, variant, Opts1).
-
+table_options(incremental, Opts0, Opts1) :-
+    !,
+    put_dict(incremental, Opts0, true, Opts1).
+table_options(dynamic, Opts0, Opts1) :-
+    !,
+    put_dict(dynamic, Opts0, true, Opts1).
+table_options(Opt, _, _) :-
+    '$domain_error'(table_option, Opt).
 
 %!  mode_check(+Moded, -TestCode)
 %
@@ -830,6 +980,8 @@ instantiated_moded_arg(Vars) :-
 %   itself. If there are multiple, this is a term s(A1,A2,...)
 
 extract_modes(ModeSpec, Head, Variant, Modes, ModedAnswer) :-
+    compound(ModeSpec),
+    !,
     compound_name_arguments(ModeSpec, Name, ModeSpecArgs),
     compound_name_arguments(Head, Name, HeadArgs),
     separate_args(ModeSpecArgs, HeadArgs, VariantArgs, Modes, ModedArgs),
@@ -842,6 +994,9 @@ extract_modes(ModeSpec, Head, Variant, Modes, ModedAnswer) :-
     ->  true
     ;   ModedAnswer =.. [s|ModedArgs]
     ).
+extract_modes(Atom, Atom, Variant, [], ModedAnswer) :-
+    atomic_list_concat([$,Atom,$,0], Variant),
+    '$tbl_trienode'(ModedAnswer).
 
 %!  separate_args(+ModeSpecArgs, +HeadArgs,
 %!		  -NoModesArgs, -Modes, -ModeArgs) is det.
@@ -902,6 +1057,12 @@ update_goal(lattice(Name/Arity), S0,S1,S2, Goal) :-
     '$must_be'(oneof(integer, lattice_arity, [3]), Arity),
     '$must_be'(atom, Name),
     Goal =.. [Name,S0,S1,S2].
+update_goal(lattice(Head), S0,S1,S2, Goal) :-
+    compound(Head),
+    !,
+    compound_name_arity(Head, Name, Arity),
+    '$must_be'(oneof(integer, lattice_arity, [3]), Arity),
+    Goal =.. [Name,S0,S1,S2].
 update_goal(lattice(Name), S0,S1,S2, Goal) :-
     !,
     '$must_be'(atom, Name),
@@ -968,17 +1129,182 @@ sum(S0, S1, S) :- S is S0+S1.
 
 
 		 /*******************************
-		 *         RENAME WORKER	*
+		 *      INCREMENTAL TABLING	*
 		 *******************************/
 
-system:term_expansion((:- table(Preds)),
-                      [ (:- multifile('$tabled'/2)),
-                        (:- multifile('$table_mode'/3)),
-                        (:- multifile('$table_update'/4))
-                      | Clauses
-                      ]) :-
+%!  '$wrap_incremental'(:Head) is det.
+%
+%   Wrap an incremental dynamic predicate to be added to the IDG.
+
+'$wrap_incremental'(Head) :-
+    abstract_goal(Head, Abstract),
+    '$wrap_predicate'(Head, incremental, Wrapped,
+                      (   '$idg_add_dyncall'(Abstract),
+                          Wrapped
+                      )),
+    '$pi_head'(PI, Head),
+    (   Head == Abstract
+    ->  prolog_listen(PI, dyn_update)
+    ;   prolog_listen(PI, dyn_update(Abstract))
+    ).
+
+abstract_goal(M:Head, M:Abstract) :-
+    compound(Head),
+    '$get_predicate_attribute'(M:Head, abstract, 1),
+    !,
+    compound_name_arity(Head, Name, Arity),
+    functor(Abstract, Name, Arity).
+abstract_goal(Head, Head).
+
+%!  dyn_update(+Action, +Context) is det.
+%
+%   Track changes to added or removed clauses. We use '$clause'/4
+%   because it works on erased clauses.
+%
+%   @tbd Add a '$clause_head'(-Head, +ClauseRef) to only decompile the
+%   head.
+
+dyn_update(_Action, ClauseRef) :-
+    (   atomic(ClauseRef)                       % avoid retractall, start(_)
+    ->  '$clause'(Head, _Body, ClauseRef, _Bindings),
+        dyn_changed_pattern(Head)
+    ;   true
+    ).
+
+dyn_update(Abstract, _, _) :-
+    dyn_changed_pattern(Abstract).
+
+dyn_changed_pattern(Term) :-
+    '$tbl_variant_table'(VTable),
+    !,
+    forall(trie_gen(VTable, Term, ATrie),
+           '$idg_changed'(ATrie)).
+dyn_changed_pattern(_).
+
+%!  '$unwrap_incremental'(:Head) is det.
+%
+%   Remove dynamic predicate incremenal forwarding,   reset the possible
+%   `abstract` property and remove possible tables.
+
+'$unwrap_incremental'(Head) :-
+    '$pi_head'(PI, Head),
+    (   unwrap_predicate(PI, incremental)
+    ->  abstract_goal(Head, Abstract),
+        (   Head == Abstract
+        ->  prolog_unlisten(PI, dyn_update)
+        ;   '$set_predicate_attribute'(Head, abstract, 0),
+            prolog_unlisten(PI, dyn_update(_))
+        ),
+        (   '$tbl_variant_table'(VariantTrie)
+        ->  forall(trie_gen(VariantTrie, Head, ATrie),
+                   '$tbl_destroy_table'(ATrie))
+        ;   true
+        )
+    ;   true
+    ).
+
+%!  reeval(+ATrie)
+%
+%   Called  if  the   table   ATrie    is   out-of-date   (has  non-zero
+%   _falsecount_). This finds all dependency paths to dynamic predicates
+%   and then evaluates the nodes in   a breath-first fashion starting at
+%   the level just above the  dynamic   predicates  and  moving upwards.
+%   Bottom up evaluation is used to   profit  from upward propagation of
+%   not-modified events that may cause the evaluation to stop early.
+
+reeval(ATrie) :-
+    findall(Path, false_path(ATrie, Path), Paths),
+    reeval_paths(Paths, ATrie).
+
+reeval_paths(BottomUp, ATrie) :-
+    is_invalid(ATrie),
+    !,
+    reeval_heads(BottomUp, ATrie, BottomUp1),
+    reeval_paths(BottomUp1, ATrie).
+reeval_paths(_, _).
+
+reeval_heads(_, ATrie, _) :-
+    \+ is_invalid(ATrie),
+    !.
+reeval_heads([], _, []).
+reeval_heads([[H]|B], ATrie, BT) :-
+    !,
+    reeval_node(H),
+    reeval_heads(B, ATrie, BT).
+reeval_heads([[]|B], ATrie, BT) :-
+    !,
+    reeval_heads(B, ATrie, BT).
+reeval_heads([[H|T]|B], ATrie, [T|BT]) :-
+    !,
+    reeval_node(H),
+    reeval_heads(B, ATrie, BT).
+
+false_path(ATrie, BottomUp) :-
+    false_path(ATrie, Path, []),
+    '$reverse'(Path, BottomUp).
+
+false_path(ATrie, [ATrie|T], Seen) :-
+    is_invalid(ATrie),
+    \+ memberchk(ATrie, Seen),
+    '$idg_edge'(ATrie, dependent, Dep),
+    (   '$tbl_table_status'(Dep, dynamic, _, _),
+        T = []
+    ;   false_path(Dep, T, [ATrie|Seen])
+    ).
+
+is_invalid(ATrie) :-
+    '$idg_falsecount'(ATrie, FalseCount),
+    FalseCount > 0.
+
+reeval_node(ATrie) :-
+    is_invalid(ATrie),
+    !,
+    '$tbl_reeval_prepare'(ATrie),
+    '$tbl_table_status'(ATrie, _, Variant, _),
+    (   '$idg_reset_current',                   % move to '$tbl_scc_save'/1?
+        setup_call_cleanup(
+            '$tbl_scc_save'(State),
+            call(Variant),
+            '$tbl_scc_restore'(State)),
+        fail
+    ;   true
+    ).
+reeval_node(_).
+
+
+		 /*******************************
+		 *      EXPAND DIRECTIVES	*
+		 *******************************/
+
+system:term_expansion((:- table(Preds)), Expansion) :-
     \+ current_prolog_flag(xref, true),
-    phrase(wrappers(Preds), Clauses).
+    prolog_load_context(module, M),
+    phrase(wrappers(Preds, M), Clauses),
+    multifile_decls(Clauses, Directives0),
+    sort(Directives0, Directives),
+    '$append'(Clauses, Directives, Expansion).
+
+multifile_decls([], []).
+multifile_decls([H0|T0], [H|T]) :-
+    multifile_decl(H0, H),
+    !,
+    multifile_decls(T0, T).
+multifile_decls([_|T0], T) :-
+    multifile_decls(T0, T).
+
+multifile_decl(M:(Head :- _Body), (:- multifile(M:Name/Arity))) :-
+    !,
+    functor(Head, Name, Arity).
+multifile_decl(M:Head, (:- multifile(M:Name/Arity))) :-
+    !,
+    functor(Head, Name, Arity).
+multifile_decl((Head :- body), (:- multifile(Name/Arity))) :-
+    !,
+    functor(Head, Name, Arity).
+multifile_decl(Head, (:- multifile(Name/Arity))) :-
+    !,
+    Head \= (:-_),
+    functor(Head, Name, Arity).
 
 
 		 /*******************************
