@@ -53,6 +53,7 @@
 
             not_exists/1,			% :Goal
             sk_not/1,				% :Goal
+            gc_tables/1,                        % -Remaining
 
             is_most_general_term/1,		% @Term
 
@@ -76,6 +77,8 @@
 :- use_module(library(debug)).
 :- use_module(library(dialect/xsb/source)).
 :- use_module(library(dialect/xsb/tables)).
+:- use_module(library(aggregate)).
+:- use_module(library(option)).
 
 /** <module> XSB Prolog compatibility layer
 
@@ -152,6 +155,7 @@ xsb_term_expansion((:- Directive), []) :-
     !.
 xsb_term_expansion((:- import Preds from From),
                    (:- xsb_import(Preds, From))).
+xsb_term_expansion((:- index(_PI, _, _)), []).  % what is tbis?
 xsb_term_expansion((:- index(_PI, _How)), []).
 xsb_term_expansion((:- index(_PI)), []).
 xsb_term_expansion((:- ti(_PI)), []).
@@ -167,6 +171,11 @@ user:goal_expansion(In, Out) :-
 
 xsb_mapped_predicate(expand_file_name(File, Expanded),
                      xsb_expand_file_name(File, Expanded)).
+xsb_mapped_predicate(abolish_module_tables(UserMod),
+                     abolish_module_tables(user)) :-
+    UserMod == usermod.
+xsb_mapped_predicate(dynamic(Spec), xsb_dynamic(Spec)) :-
+    Spec = as(_,_).
 
 xsb_inlined_goal(fail_if(P), \+(P)).
 
@@ -450,25 +459,59 @@ clear_compiler_option(xpp_on).
 %
 %       :- import p/1 from x.
 %       :- dynamic p/1.
+%
+%   This also deals with ``:- dynamic Spec as Flags``.
 
 xsb_dynamic(M:Preds) :-
-    xsb_dynamic_(Preds, M).
+    xsb_dynamic_(Preds, M, []).
 
-xsb_dynamic_(Preds, _M) :-
+xsb_dynamic_(Preds, _M, _) :-
     var(Preds),
     !,
     instantiation_error(Preds).
-xsb_dynamic_((A,B), M) :-
+xsb_dynamic_(Spec as Flags, M, Options0) :-
     !,
-    xsb_dynamic_(A, M),
-    xsb_dynamic_(B, M).
-xsb_dynamic_(Name/Arity, M) :-
+    dyn_flags(Flags, Options0, Options),
+    xsb_dynamic_(Spec, M, Options).
+xsb_dynamic_((A,B), M, Options) :-
+    !,
+    xsb_dynamic_(A, M, Options),
+    xsb_dynamic_(B, M, Options).
+/* JW: not really clear why this is needed/desirable.
+xsb_dynamic_(Name/Arity, M, Options) :-
     functor(Head, Name, Arity),
     '$get_predicate_attribute'(M:Head, imported, M2), % predicate_property/2 requires
     !,                                                % P to be defined.
-    dynamic(M2:Name/Arity).
-xsb_dynamic_(PI, M) :-
-    dynamic(M:PI).
+    dynamic([M2:Name/Arity], Options).
+*/
+xsb_dynamic_(PI, M, Options) :-
+    dynamic([M:PI], Options).
+
+dyn_flags(Var, _, _) :-
+    var(Var),
+    !,
+    instantiation_error(Var).
+dyn_flags((A,B), Options0, Options) :-
+    !,
+    dyn_flags(A, Options0, Options1),
+    dyn_flags(B, Options1, Options).
+dyn_flags(incremental, Options0, Options) :-
+    !,
+    merge_options([incremental(true)], Options0, Options).
+dyn_flags(opaque, Options0, Options) :-
+    !,
+    merge_options([incremental(false)], Options0, Options).
+dyn_flags(shared, Options0, Options) :-
+    !,
+    merge_options([thread(shared)], Options0, Options).
+dyn_flags(private, Options0, Options) :-
+    !,
+    merge_options([thread(local)], Options0, Options).
+dyn_flags(abstract(Level), Options0, Options) :-
+    !,
+    merge_options([abstract(Level)], Options0, Options).
+dyn_flags(Flag, _, _) :-
+    domain_error(xsb_dynamic_flag, Flag).
 
 
 		 /*******************************
@@ -500,6 +543,25 @@ not_exists(P) :-
 sk_not(P) :-
     not_exists(P).
 
+%!  gc_tables(-Remaining) is det.
+%
+%   The table abolish predicates leave  the   actual  destruction of the
+%   tables to the atom  garbage  collector   to  avoid  deleting  active
+%   tables. This predicate runs garbage_collect_atoms/0   and counts the
+%   remaining erased tables.
+%
+%   @compat Due to the heuristic nature of garbage_collect_atoms/0, not
+%   all tables may be reclaimed immediately.
+
+gc_tables(Remaining) :-
+    garbage_collect_atoms,
+    aggregate_all(count, remaining_table(_), Remaining).
+
+remaining_table(Trie) :-
+    current_blob(Trie, trie),
+    '$is_answer_trie'(Trie),
+    '$atom_references'(Trie, 0).
+
 
 %!  is_most_general_term(@X) is semidet.
 %
@@ -557,8 +619,8 @@ walltime(Seconds) :-
     Seconds is Now - Epoch.
 
 
-%!  path_sysop(+Op, ?Value)
-%!  path_sysop(+Op, ?Arg1, ?Arg2)
+%!  path_sysop(+Op, ?Value) is semidet.
+%!  path_sysop(+Op, ?Arg1, ?Arg2) is semidet.
 %
 %   Unified interface to the  operations  on   files.  All  these  calls
 %   succeed iff the corresponding system call succeeds.
