@@ -1766,6 +1766,10 @@ valid_meta_decl(I) :- integer(I), between(0,9,I).
 
 %!  colourise_table_declarations(+Term, +TB, +Pos)
 
+colourise_table_declarations(Term, TB, parentheses_term_position(PO,PC,Pos)) :-
+    !,
+    colour_item(parentheses, TB, PO-PC),
+    colourise_table_declarations(Term, TB, Pos).
 colourise_table_declarations((Head,Tail), TB,
                              term_position(_,_,_,_,[PH,PT])) :-
     !,
@@ -1788,6 +1792,11 @@ colourise_table_declarations(Goal, TB, term_position(_F,_T,FF,FT,ArgPos)) :-
     goal_classification(TB, Goal, [], Class),
     colour_item(goal(Class, Goal), TB, FF-FT),
     colourise_table_modes(Args, TB, ArgPos).
+colourise_table_declarations(Goal, TB, Pos) :-
+    atom(Goal),
+    !,
+    goal_classification(TB, Goal, [], Class),
+    colour_item(goal(Class, Goal), TB, Pos).
 
 colourise_table_modes([], _, _).
 colourise_table_modes([H|T], TB, [PH|PT]) :-
@@ -1835,6 +1844,14 @@ table_moded_call(Atom/Arity, Arity, TB,
     colour_item(goal(Class, Head), TB, NP),
     colour_item(predicate_indicator, TB, FF-FT),
     colour_item(arity, TB, AP).
+table_moded_call(Head, Arity, TB, Pos) :-
+    Pos = term_position(_,_,FF,FT,_),
+    compound(Head),
+    !,
+    compound_name_arity(Head, _Name, Arity),
+    goal_classification(TB, Head, [], Class),
+    colour_item(goal(Class, Head), TB, FF-FT),
+    colourise_term_args(Head, TB, Pos).
 table_moded_call(_, _, TB, Pos) :-
     colour_item(type_error(predicate_name_or_indicator), TB, Pos).
 
@@ -1853,6 +1870,7 @@ colourise_table_options(_, TB, Pos) :-
 
 valid_table_option(subsumptive).
 valid_table_option(variant).
+valid_table_option(incremental).
 
 %!  colourise_op_declaration(Op, TB, Pos) is det.
 
