@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2017, University of Amsterdam
+    Copyright (c)  2011-2019, University of Amsterdam
                               VU University Amsterdam
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -79,6 +80,7 @@ too much.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static void setArgvPrologFlag(const char *flag, int argc, char **argv);
+static void setTmpDirPrologFlag(void);
 static void setTZPrologFlag(void);
 static void setVersionPrologFlag(void);
 static void initPrologFlagTable(void);
@@ -732,7 +734,14 @@ set_prolog_flag_unlocked(term_t key, term_t value, int flags)
       { debugstatus.showContext = val;
 #ifdef O_PLMT
       } else if ( k == ATOM_threads )
-      { if ( !(rval = enableThreads(val)) )
+      { if ( val )
+	{ rval = enableThreads(val);
+	  PL_LOCK(L_PLFLAG);
+	} else
+	{ PL_UNLOCK(L_PLFLAG);
+	  rval = enableThreads(val);
+	}
+	if ( !rval )
 	  break;			/* don't change value */
 #endif
       } else if ( k == ATOM_tty_control )
@@ -1266,6 +1275,9 @@ pl_prolog_flag(term_t name, term_t value, control_t h)
 #ifndef SO_PATH
 #define SO_PATH "LD_LIBRARY_PATH"
 #endif
+#ifndef C_LIBPLSO
+#define C_LIBPLSO ""
+#endif
 
 static void
 initPrologFlagTable(void)
@@ -1284,6 +1296,9 @@ initPrologFlags(void)
   setPrologFlag("arch", FT_ATOM|FF_READONLY, PLARCH);
 #if __WINDOWS__
   setPrologFlag("windows",	FT_BOOL|FF_READONLY, TRUE, 0);
+  const char *wine_version;
+  if ( (wine_version=PL_w32_running_under_wine()) )
+    setPrologFlag("wine_version", FT_ATOM|FF_READONLY, wine_version, 0);
 #endif
 #if O_XOS
   setPrologFlag("win_file_access_check", FT_ATOM,
@@ -1311,9 +1326,13 @@ initPrologFlags(void)
 		PLFLAG_WARN_OVERRIDE_IMPLICIT_IMPORT);
   setPrologFlag("c_cc",	     FT_ATOM, C_CC);
   setPrologFlag("c_libs",    FT_ATOM, C_LIBS);
+#ifdef C_LIBDIR
+  setPrologFlag("c_libdir",  FT_ATOM, C_LIBDIR);
+#endif
   setPrologFlag("c_libplso", FT_ATOM, C_LIBPLSO);
   setPrologFlag("c_ldflags", FT_ATOM, C_LDFLAGS);
   setPrologFlag("c_cflags",  FT_ATOM, C_CFLAGS);
+  setPrologFlag("tmp_dir", FT_ATOM, SWIPL_TMP_DIR);
 #if defined(O_LARGEFILES) || SIZEOF_LONG == 8
   setPrologFlag("large_files", FT_BOOL|FF_READONLY, TRUE, 0);
 #endif
@@ -1335,9 +1354,10 @@ initPrologFlags(void)
   setPrologFlag("pipe", FT_BOOL, TRUE, 0);
 #endif
 #ifdef O_PLMT
-  setPrologFlag("threads",	FT_BOOL|FF_READONLY, TRUE, 0);
+  setPrologFlag("threads",	FT_BOOL, !GD->options.nothreads, 0);
   setPrologFlag("system_thread_id", FT_INTEGER|FF_READONLY, 0, 0);
   setPrologFlag("gc_thread",    FT_BOOL,
+		!GD->options.nothreads &&
 		truePrologFlag(PLFLAG_GCTHREAD), PLFLAG_GCTHREAD);
 #else
   setPrologFlag("threads",	FT_BOOL|FF_READONLY, FALSE, 0);
@@ -1396,6 +1416,7 @@ initPrologFlags(void)
 		GD->options.traditional ? "codes" : "string");
   setPrologFlag("back_quotes", FT_ATOM,
 		GD->options.traditional ? "symbol_char" : "codes");
+  setPrologFlag("portable_vmi", FT_BOOL, TRUE, PLFLAG_PORTABLE_VMI);
   setPrologFlag("traditional", FT_BOOL|FF_READONLY, GD->options.traditional, 0);
   setPrologFlag("unknown", FT_ATOM, "error");
   setPrologFlag("debug", FT_BOOL, FALSE, 0);
@@ -1413,11 +1434,24 @@ initPrologFlags(void)
   setPrologFlag("toplevel_prompt", FT_ATOM, "~m~d~l~! ?- ");
   setPrologFlag("file_name_variables", FT_BOOL, FALSE, PLFLAG_FILEVARS);
   setPrologFlag("fileerrors", FT_BOOL, TRUE, PLFLAG_FILEERRORS);
+#ifdef O_DEBUG
+  setPrologFlag("prolog_debug", FT_BOOL|FF_READONLY, TRUE, 0);
+#endif
+#ifdef __EMSCRIPTEN__
+  setPrologFlag("emscripten", FT_BOOL|FF_READONLY, TRUE, 0);
+#else
 #ifdef __unix__
   setPrologFlag("unix", FT_BOOL|FF_READONLY, TRUE, 0);
 #endif
 #ifdef __APPLE__
   setPrologFlag("apple", FT_BOOL|FF_READONLY, TRUE, 0);
+#endif
+#ifdef __ANDROID__
+  setPrologFlag("android", FT_BOOL|FF_READONLY, TRUE, 0);
+# ifdef __ANDROID_API__
+  setPrologFlag("android_api",FT_INTEGER|FF_READONLY, __ANDROID_API__);
+# endif
+#endif
 #endif
 
   setPrologFlag("encoding", FT_ATOM, stringAtom(encoding_to_atom(LD->encoding)));
@@ -1441,7 +1475,9 @@ initPrologFlags(void)
 #ifdef O_MITIGATE_SPECTRE
   setPrologFlag("mitigate_spectre", FT_BOOL, FALSE, PLFLAG_MITIGATE_SPECTRE);
 #endif
+  setPrologFlag("posix_shell", FT_ATOM, POSIX_SHELL);
 
+  setTmpDirPrologFlag();
   setTZPrologFlag();
   setOSPrologFlags();
   setVersionPrologFlag();
@@ -1450,6 +1486,23 @@ initPrologFlags(void)
 }
 
 
+static void
+setTmpDirPrologFlag(void)
+ { char envbuf[MAXPATHLEN];
+   char *td = NULL;
+
+#ifdef __unix__
+   td=Getenv("TMP", envbuf, sizeof(envbuf));
+#elif __WINDOWS__
+   td=Getenv("TEMP", envbuf, sizeof(envbuf));
+#endif
+
+   if (td == (char *) NULL)
+     td = SWIPL_TMP_DIR;
+
+   setPrologFlag("tmp_dir", FT_ATOM, td);
+}
+
 static void
 setArgvPrologFlag(const char *flag, int argc, char **argv)
 { GET_LD
