@@ -188,7 +188,12 @@ const debug_topic debug_topics[] =
   DEBUG_TOPIC(MSG_TABLING_DELAY_VAR),
   DEBUG_TOPIC(MSG_TABLING_AC),
   DEBUG_TOPIC(MSG_TABLING_MERGE),
+  DEBUG_TOPIC(MSG_TABLING_VTRIE_DEPENDENCIES),
+  DEBUG_TOPIC(MSG_TABLING_IDG),
+  DEBUG_TOPIC(MSG_TABLING_IDG_CHANGED),
+  DEBUG_TOPIC(MSG_TABLING_IDG_REEVAL),
 
+  DEBUG_TOPIC(TABLING_NO_EARLY_COMPLETION),
   DEBUG_TOPIC(TABLING_NO_SIMPLIFY),
   DEBUG_TOPIC(TABLING_NO_AC),
 
