@@ -862,10 +862,12 @@ with one operation, it turns out to be faster as well.
 #define HIDE_CHILDS		(0x00800000) /* Hide children from tracer */
 #define SPY_ME			(0x01000000) /* Spy point placed */
 #define TRACE_ME		(0x02000000) /* Can be debugged */
-#define TRACE_CALL		(0x04000000) /* Trace calls */
-#define TRACE_REDO		(0x08000000) /* Trace redo */
-#define TRACE_EXIT		(0x10000000) /* Trace edit */
-#define TRACE_FAIL		(0x20000000) /* Trace fail */
+#define P_INCREMENTAL		(0x04000000) /* Incremental tabling */
+#define P_ABSTRACT		(0x08000000) /* Incremental tabling */
+/** unused
+#define			(0x10000000)
+#define			(0x20000000)
+*/
 #define FILE_ASSIGNED		(0x40000000) /* Is assigned to a file */
 #define P_REDEFINED		(0x80000000) /* Overrules a definition */
 #define PROC_DEFINED		(P_DYNAMIC|P_FOREIGN|P_MULTIFILE|P_DISCONTIGUOUS)
@@ -1453,6 +1455,7 @@ struct definition
   unsigned int  shared;			/* #procedures sharing this def */
   struct linger_list  *lingering;	/* Assocated lingering objects */
   gen_t		last_modified;		/* Generation I was last modified */
+  struct event_list *events;		/* Forward update events */
 #ifdef O_PROF_PENTIUM
   int		prof_index;		/* index in profiling */
   char	       *prof_name;		/* name in profiling */
@@ -1968,26 +1971,6 @@ typedef struct
 #define SIG_TUNE_GC	  (SIG_PROLOG_OFFSET+5)
 
 
-		 /*******************************
-		 *	      EVENTS		*
-		 *******************************/
-
-typedef enum pl_event_type
-{ PLEV_ABORT,				/* Execution aborted */
-  PLEV_ERASED_CLAUSE,			/* clause was erased */
-  PLEV_ERASED_RECORD,			/* record was erased */
-  PLEV_DEBUGGING,			/* changed debugging mode */
-  PLEV_TRACING,				/* changed tracing mode */
-  PLEV_SPY,				/* changed spypoint */
-  PLEV_BREAK,				/* a break-point was set */
-  PLEV_BREAK_EXISTS,			/* existing breakpoint */
-  PLEV_NOBREAK,				/* a break-point was cleared */
-  PLEV_GCNOBREAK,			/* cleared due to clause GC */
-  PLEV_FRAMEFINISHED,			/* A watched frame was discarded */
-  PL_EV_THREADFINISHED			/* A thread has finished */
-} pl_event_type;
-
-
 		 /*******************************
 		 *	       COMPARE		*
 		 *******************************/
@@ -2264,7 +2247,6 @@ typedef struct
 #define PROCEDURE_reset3		(GD->procedures.reset3)
 #define PROCEDURE_true0			(GD->procedures.true0)
 #define PROCEDURE_fail0			(GD->procedures.fail0)
-#define PROCEDURE_event_hook1		(GD->procedures.event_hook1)
 #define PROCEDURE_print_message2	(GD->procedures.print_message2)
 #define PROCEDURE_dcall1		(GD->procedures.dcall1)
 #define PROCEDURE_setup_call_catcher_cleanup4 \
