@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2017, University of Amsterdam
+    Copyright (c)  1985-2019, University of Amsterdam
                               VU University Amsterdam
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -36,6 +37,7 @@
 /*#define O_DEBUG 1*/
 #include "pl-incl.h"
 #include "pl-dbref.h"
+#include "pl-event.h"
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 General  handling  of  procedures:  creation;  adding/removing  clauses;
@@ -270,6 +272,8 @@ resetProcedure(Procedure proc, bool isnew)
   if ( stringAtom(def->functor->name)[0] != '$' )
     set(def, TRACE_ME);
   def->impl.clauses.number_of_clauses = 0;
+  if ( def->events )
+    destroy_event_list(&def->events);
 
   if ( isnew )
   { deleteIndexes(&def->impl.clauses, TRUE);
@@ -1236,6 +1240,11 @@ assertProcedure(Procedure proc, Clause clause, ClauseRef where ARG_LD)
   release_def(def);
   DEBUG(CHK_SECURE, checkDefinition(def));
   UNLOCKDEF(def);
+  if ( def->events &&
+       !predicate_update_event(def,
+			       where == CL_START ? ATOM_asserta : ATOM_assertz,
+			       clause PASS_LD) )
+    return NULL;
 
   return cref;
 }
@@ -1397,6 +1406,9 @@ retractClauseDefinition(Definition def, Clause clause)
 
   registerDirtyDefinition(def PASS_LD);
 
+  if ( def->events )
+    return predicate_update_event(def, ATOM_retract, clause PASS_LD);
+
   return TRUE;
 }
 
@@ -1446,22 +1458,11 @@ freeClause(Clause c)
 }
 
 
+
 static int WUNUSED			/* FALSE if there was an error */
 announceErasedClause(Clause clause)
-{
-#if O_DEBUGGER
-  int rc;
-  Definition def = clause->predicate;
-
-  rc = clearBreakPointsClause(clause) >= 0;
-  if ( PROCEDURE_event_hook1 &&
-       def != PROCEDURE_event_hook1->definition )
-    rc = callEventHook(PLEV_ERASED_CLAUSE, clause) && rc;
-
-  return rc;
-#endif
-
-  return TRUE;
+{ return ( clearBreakPointsClause(clause) >= 0 &&
+	   callEventHook(PLEV_ERASED_CLAUSE, clause) );
 }
 
 
@@ -2610,6 +2611,7 @@ pl_retractall(term_t head)
   Word argv;
   int allvars = TRUE;
   fid_t fid;
+  int rc = TRUE;
 
   if ( !get_procedure(head, &proc, thehead, GP_CREATE) )
     fail;
@@ -2625,6 +2627,9 @@ pl_retractall(term_t head)
     succeed;				/* nothing to retract */
   }
 
+  if ( !retractall_event(def, thehead, FUNCTOR_start1 PASS_LD) )
+    return FALSE;
+
   argv = valTermRef(thehead);
   deRef(argv);
   if ( isTerm(*argv) )
@@ -2637,7 +2642,6 @@ pl_retractall(term_t head)
     argv = NULL;
   }
 
-  startCritical;
   enterDefinition(def);
   setGenerationFrameVal(environment_frame, pushPredicateAccess(def));
   fid = PL_open_foreign_frame();
@@ -2652,7 +2656,8 @@ pl_retractall(term_t head)
     acquire_def(def);
     for(cref = def->impl.clauses.first_clause; cref; cref = cref->next)
     { if ( visibleClauseCNT(cref->value.clause, gen) )
-      { retractClauseDefinition(def, cref->value.clause);
+      { if ( !(rc=retractClauseDefinition(def, cref->value.clause)) )
+	  break;
       }
     }
     release_def(def);
@@ -2660,22 +2665,23 @@ pl_retractall(term_t head)
   { struct clause_choice chp;
 
     if ( !(cref = firstClause(argv, environment_frame, def, &chp PASS_LD)) )
-    { int rc = endCritical;
-      popPredicateAccess(def);
+    { popPredicateAccess(def);
       leaveDefinition(def);
-      return rc;
+      return TRUE;
     }
 
     while( cref )
     { if ( decompileHead(cref->value.clause, thehead) )
-	retractClauseDefinition(def, cref->value.clause);
+      { if ( !(rc=retractClauseDefinition(def, cref->value.clause)) )
+	  break;
+      }
 
       PL_rewind_foreign_frame(fid);
 
       if ( !chp.cref )
       { popPredicateAccess(def);
 	leaveDefinition(def);
-	return endCritical;
+	return TRUE;
       }
 
       if ( argv )				/* may be shifted */
@@ -2693,7 +2699,10 @@ pl_retractall(term_t head)
 	checkDefinition(def);
 	UNLOCKDEF(def));
 
-  return endCritical;
+  if ( rc )
+    rc = retractall_event(def, thehead, FUNCTOR_end1 PASS_LD);
+
+  return rc;
 }
 
 		/********************************
@@ -2762,8 +2771,6 @@ typedef struct patt_mask
   unsigned int  mask;
 } patt_mask;
 
-#define TRACE_ANY (TRACE_CALL|TRACE_REDO|TRACE_EXIT|TRACE_FAIL)
-
 static const patt_mask patt_masks[] =
 { { ATOM_dynamic,	   P_DYNAMIC },
   { ATOM_multifile,	   P_MULTIFILE },
@@ -2771,12 +2778,9 @@ static const patt_mask patt_masks[] =
   { ATOM_system,	   P_LOCKED },		/* compatibility */
   { ATOM_spy,		   SPY_ME },
   { ATOM_tabled,	   P_TABLED },
+  { ATOM_incremental,	   P_INCREMENTAL },
+  { ATOM_abstract,	   P_ABSTRACT },
   { ATOM_trace,		   TRACE_ME },
-  { ATOM_trace_call,	   TRACE_CALL },
-  { ATOM_trace_redo,	   TRACE_REDO },
-  { ATOM_trace_exit,	   TRACE_EXIT },
-  { ATOM_trace_fail,	   TRACE_FAIL },
-  { ATOM_trace_any,	   TRACE_ANY },
   { ATOM_hide_childs,	   HIDE_CHILDS },
   { ATOM_transparent,	   P_TRANSPARENT },
   { ATOM_discontiguous,	   P_DISCONTIGUOUS },
@@ -3078,7 +3082,7 @@ pl_set_predicate_attribute(term_t pred, term_t what, term_t value)
        !(att = attribute_mask(key)) )
     return FALSE;
 
-  if ( att & (TRACE_ANY|SPY_ME) )
+  if ( (att&SPY_ME) )
   { if ( !get_procedure(pred, &proc, 0, GP_RESOLVE) )
       fail;
   } else
