@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2017, University of Amsterdam
+    Copyright (c)  1985-2019, University of Amsterdam
                               VU University Amsterdam
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -648,7 +649,7 @@ static void
 hupHandler(int sig)
 { (void)sig;
 
-  PL_halt(2);
+  PL_halt(128+sig);
 }
 #endif
 
@@ -663,12 +664,28 @@ static void
 terminate_handler(int sig)
 { signal(sig, SIG_DFL);
 
-  run_on_halt(&GD->os.exit_hooks, 3);
+  run_on_halt(&GD->os.exit_hooks, 128+sig);
 
 #if defined(HAVE_KILL) && defined(HAVE_GETPID)
   kill(getpid(), sig);
 #else
-  exit(3);
+  switch( sig )
+  {
+#ifdef SIGTERM
+    case SIGTERM:
+      exit(128+SIGTERM);
+#endif
+#ifdef SIGQUIT
+    case SIGQUIT:
+      exit(128+SIGQUIT);
+#endif
+#ifdef SIGABRT
+    case SIGABRT:
+      abort();
+#endif
+    default:
+      assert(0); /* not reached */
+  }
 #endif
 }
 
@@ -1348,6 +1365,7 @@ emptyStacks(void)
     DEBUG(3, Sdprintf("attvar.tail at %p\n", valTermRef(LD->attvar.tail)));
 #endif
     LD->tabling.delay_list  = init_delay_list();
+    LD->tabling.idg_current = PL_new_term_ref();
 #ifdef O_GVAR
     destroyGlobalVars();
 #endif
