@@ -111,6 +111,8 @@ typedef struct worklist
   unsigned	neg_delayed : 1;	/* Negative node was delayed */
   unsigned	has_answers : 1;	/* At least one unconditional answer */
   unsigned	answer_completed : 1;	/* Is answer completed */
+  unsigned	depend_abolish : 1;	/* Scheduled for depending abolish */
+  unsigned	abolish_on_complete : 1;/* Abolish the table on completion */
   size_t	undefined;		/* #undefined answers */
 
   tbl_component*component;		/* component I belong to */
@@ -169,12 +171,26 @@ typedef struct delay_info
 } delay_info;
 
 
+		 /*******************************
+		 *	       IDG		*
+		 *******************************/
+
+typedef struct idg_node
+{ Table		affected;		/* parent nodes */
+  Table		dependent;		/* childs */
+  trie	       *atrie;			/* answer trie */
+  size_t	answer_count;		/* #answers in previous complete state */
+  unsigned	new_answer : 1;		/* Update generated a new answer */
+  unsigned	reevaluating : 1;	/* currently re-evaluating */
+  int		falsecount;		/* Invalidate count */
+} idg_node;
+
+
 		 /*******************************
 		 *	     PROTOTYPES		*
 		 *******************************/
 
 COMMON(void)	clearThreadTablingData(PL_local_data_t *ld);
 COMMON(term_t)	init_delay_list(void);
-COMMON(void)	destroy_delay_info(delay_info *di);
 
 #endif /*_PL_TABLING_H*/
