@@ -37,6 +37,7 @@
 #include "pl-trie.h"
 #include "pl-tabling.h"
 #include "pl-indirect.h"
+#define NO_AC_TERM_WALK 1
 #define AC_TERM_WALK_POP 1
 #include "pl-termwalk.c"
 
@@ -169,7 +170,6 @@ static PL_blob_t trie_blob =
 		 *******************************/
 
 static trie_node       *new_trie_node(trie *trie, word key);
-static void		clear_vars(Word k, size_t var_number ARG_LD);
 static void		destroy_node(trie *trie, trie_node *n);
 static void		clear_node(trie *trie, trie_node *n, int dealloc);
 static inline void	release_value(word value);
@@ -294,8 +294,6 @@ next:
   release_key(n->key);
   if ( n->value )
     release_value(n->value);
-  if ( n->data.delayinfo )		/* hacky */
-    destroy_delay_info(n->data.delayinfo);
 
   if ( dealloc )
   { ATOMIC_DEC(&trie->node_count);
@@ -494,12 +492,13 @@ prune_error(trie *trie, trie_node *node ARG_LD)
 
 
 int
-trie_lookup(trie *trie, trie_node **nodep, Word k, int add ARG_LD)
+trie_lookup(trie *trie, trie_node **nodep, Word k, int add, TmpBuffer vars ARG_LD)
 { term_agenda_P agenda;
   trie_node *node = &trie->root;
   size_t var_number = 0;
   int rc = TRUE;
   size_t compounds = 0;
+  tmp_buffer varb;
 
   TRIE_STAT_INC(trie, lookups);
 
@@ -525,7 +524,13 @@ trie_lookup(trie *trie, trie_node **nodep, Word k, int add ARG_LD)
     switch( tag(w) )
     { case TAG_VAR:
 	if ( isVar(w) )
-	  *p = w = ((((word)++var_number))<<LMASK_BITS)|TAG_VAR;
+	{ if ( var_number++ == 0 && !vars )
+	  { vars = &varb;
+	    initBuffer(vars);
+	  }
+	  addBuffer(vars, p, Word);
+	  *p = w = ((((word)var_number))<<LMASK_BITS)|TAG_VAR;
+	}
         node = follow_node(trie, node, w, add PASS_LD);
 	break;
       case TAG_ATTVAR:
@@ -563,7 +568,18 @@ trie_lookup(trie *trie, trie_node **nodep, Word k, int add ARG_LD)
     }
   }
   clearTermAgenda_P(&agenda);
-  clear_vars(k, var_number PASS_LD);
+
+  if ( var_number )
+  { Word *pp = baseBuffer(vars, Word);
+    Word *ep = topBuffer(vars, Word);
+
+    for(; pp < ep; pp++)
+    { Word vp = *pp;
+      setVar(*vp);
+    }
+    if ( vars == &varb )
+      discardBuffer(vars);
+  }
 
   if ( rc == TRUE )
   { if ( node )
@@ -576,48 +592,14 @@ trie_lookup(trie *trie, trie_node **nodep, Word k, int add ARG_LD)
 }
 
 
-static void
-clear_vars(Word k, size_t var_number ARG_LD)
-{ if ( var_number > 0 )
-  { term_agenda agenda;
-    Word p;
-
-    initTermAgenda(&agenda, 1, k);
-    while( var_number > 0 && (p=nextTermAgenda(&agenda)) )
-    { word w = *p;
-
-      switch( tag(w) )
-      { case TAG_VAR:
-	{ if ( !isVar(*p) )
-	  { setVar(*p);
-	    --var_number;
-	  }
-	  break;
-	}
-        case TAG_COMPOUND:
-	{ Functor f = valueTerm(w);
-	  int arity = arityFunctor(f->definition);
-
-	  pushWorkAgenda(&agenda, arity, f->arguments);
-	  break;
-	}
-      }
-    }
-    clearTermAgenda(&agenda);
-
-    assert(var_number == 0);
-  }
-}
-
-
 trie *
-get_trie_form_node(trie_node *node)
+get_trie_from_node(trie_node *node)
 { trie *trie_ptr;
 
   for( ; node->parent; node = node->parent )
     ;
   trie_ptr = (trie *)((char*)node - offsetof(trie, root));
-  assert(trie_ptr->magic == TRIE_MAGIC);
+  assert(trie_ptr->magic == TRIE_MAGIC || trie_ptr->magic == TRIE_CMAGIC);
 
   return trie_ptr;
 }
@@ -648,7 +630,7 @@ unify_trie_term(trie_node *node, term_t term ARG_LD)
   size_t kc = 0;
   int rc = TRUE;
   trie *trie_ptr;
-  fid_t fid;
+  mark m;
 						/* get the keys */
   for( ; node->parent; node = node->parent )
   { if ( kc == keys_allocated )
@@ -673,20 +655,20 @@ unify_trie_term(trie_node *node, term_t term ARG_LD)
   trie_ptr = (trie *)((char*)node - offsetof(trie, root));
   assert(trie_ptr->magic == TRIE_MAGIC);
 
-  fid = PL_open_foreign_frame();
   for(;;)
   { ukey_state ustate;
     size_t i;
 
   retry:
+    Mark(m);
     init_ukey_state(&ustate, trie_ptr, valTermRef(term));
     for(i=kc; i-- > 0; )
     { if ( (rc=unify_key(&ustate, keys[i] PASS_LD)) != TRUE )
       { destroy_ukey_state(&ustate);
 	if ( rc == FALSE )
 	  goto out;
-	PL_rewind_foreign_frame(fid);
-	if ( makeMoreStackSpace(rc, ALLOW_GC|ALLOW_SHIFT) )
+	Undo(m);
+	if ( makeMoreStackSpace(rc, ALLOW_GC) )
 	  goto retry;
       }
     }
@@ -695,7 +677,6 @@ unify_trie_term(trie_node *node, term_t term ARG_LD)
   }
 
 out:
-  PL_close_foreign_frame(fid);
   if ( keys != fast )
     free(keys);
 
@@ -826,7 +807,6 @@ symbol_trie(atom_t symbol)
       return ref->trie;
   }
 
-  assert(0);
   return NULL;
 }
 
@@ -859,6 +839,22 @@ get_trie(term_t t, trie **tp)
 }
 
 
+int
+get_trie_noex(term_t t, trie **tp)
+{ void *data;
+  PL_blob_t *type;
+
+  if ( PL_get_blob(t, &data, NULL, &type) && type == &trie_blob )
+  { tref *ref = data;
+
+    *tp = ref->trie;
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
+
 int
 trie_error(int rc, term_t culprit)
 { switch(rc)
@@ -1017,15 +1013,32 @@ set_trie_value(trie *trie, trie_node *node, term_t value ARG_LD)
   return TRUE;
 }
 
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Delete a node from the trie. There are   two options: (1) simply set the
+value to 0 or (2), prune the branch   leading to this cell upwards until
+we find another existing node.
+
+TBD: create some sort of  lingering   mechanism  to allow for concurrent
+delete and gen_trie(). More modest: link up the deleted nodes and remove
+them after the references drop to 0.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 void
 trie_delete(trie *trie, trie_node *node, int prune)
 { if ( node->value )
-  { if ( prune )
+  { if ( prune && trie->references == 0 )
     { prune_node(trie, node);
     } else
-    { word v = node->value;
-      node->value = 0;
-      release_value(v);
+    { word v;
+
+      if ( trie->release_node )
+	(*trie->release_node)(trie, node);
+
+      if ( (v=node->value) )
+      { node->value = 0;
+	release_value(v);
+      }
     }
     ATOMIC_DEC(&trie->value_count);
   }
@@ -1053,7 +1066,7 @@ trie_insert(term_t Trie, term_t Key, term_t Value, trie_node **nodep,
 
     kp	= valTermRef(Key);
 
-    if ( (rc=trie_lookup(trie, &node, kp, TRUE PASS_LD)) == TRUE )
+    if ( (rc=trie_lookup(trie, &node, kp, TRUE, NULL PASS_LD)) == TRUE )
     { word val = intern_value(Value PASS_LD);
 
       if ( nodep )
@@ -1161,11 +1174,10 @@ PRED_IMPL("trie_delete", 3, trie_delete, 0)
 
     kp = valTermRef(A2);
 
-    if ( (rc=trie_lookup(trie, &node, kp, FALSE PASS_LD)) == TRUE )
+    if ( (rc=trie_lookup(trie, &node, kp, FALSE, NULL PASS_LD)) == TRUE )
     { if ( node->value )
       { if ( unify_value(A3, node->value PASS_LD) )
-	{ prune_node(trie, node);
-	  ATOMIC_DEC(&trie->value_count);
+	{ trie_delete(trie, node, TRUE);
 	  return TRUE;
 	}
       }
@@ -1191,7 +1203,7 @@ PRED_IMPL("trie_lookup", 3, trie_lookup, 0)
 
     kp = valTermRef(A2);
 
-    if ( (rc=trie_lookup(trie, &node, kp, FALSE PASS_LD)) == TRUE )
+    if ( (rc=trie_lookup(trie, &node, kp, FALSE, NULL PASS_LD)) == TRUE )
     { if ( node->value )
 	return unify_value(A3, node->value PASS_LD);
       return FALSE;
@@ -1563,24 +1575,24 @@ add_choice(trie_gen_state *state, descent_state *dstate, trie_node *node ARG_LD)
   if ( children.any )
   { switch( children.any->type )
     { case TN_KEY:
-      if ( !has_key ||
-	   k == children.key->key ||
-	   tagex(children.key->key) == TAG_VAR ||
-	   IS_TRIE_KEY_POP(children.key->key) )
-      {	word key   = children.key->key;
+	if ( !has_key ||
+	     k == children.key->key ||
+	     tagex(children.key->key) == TAG_VAR ||
+	     IS_TRIE_KEY_POP(children.key->key) )
+	{ word key   = children.key->key;
 
-	ch = allocFromBuffer(&state->choicepoints, sizeof(*ch));
-	ch->key    = key;
-	ch->child  = children.key->child;
-        ch->choice.any = NULL;
+	  ch = allocFromBuffer(&state->choicepoints, sizeof(*ch));
+	  ch->key    = key;
+	  ch->child  = children.key->child;
+	  ch->choice.any = NULL;
 
-	if ( IS_TRIE_KEY_POP(children.key->key) && dstate->compound )
-	{ popSegStack(&dstate->stack, &dstate->term, Word);
-	  //Sdprintf("Popped %p\n", dstate->term);
-	}
-	break;
-      } else
-	return NULL;
+	  if ( IS_TRIE_KEY_POP(children.key->key) && dstate->compound )
+	  { popSegStack(&dstate->stack, &dstate->term, Word);
+	    //Sdprintf("Popped %p\n", dstate->term);
+	  }
+	  break;
+	} else
+	  return NULL;
       case TN_HASHED:
 	if ( has_key && children.hash->var_keys == 0 )
 	{ trie_node *child;
@@ -1659,11 +1671,11 @@ next_choice0(trie_gen_state *state, descent_state *dstate ARG_LD)
     ch--;
   }
 
-  return FALSE;
+  return NULL;
 }
 
 
-static int
+static trie_choice *
 next_choice(trie_gen_state *state ARG_LD)
 { trie_choice *ch;
   descent_state dstate;
@@ -1675,7 +1687,15 @@ next_choice(trie_gen_state *state ARG_LD)
   { ch = next_choice0(state, &dstate PASS_LD);
   } while (ch && ch->child->value == 0);
 
-  return ch != NULL;
+  return ch;
+}
+
+
+static trie_node *
+gen_state_leaf(trie_gen_state *state)
+{ trie_choice *top = top_choice(state);
+
+  return top[-1].child;
 }
 
 
@@ -1715,7 +1735,6 @@ trie_gen(term_t Trie, term_t Key, term_t Value,
   trie_gen_state state_buf;
   trie_gen_state *state;
   trie_node *n;
-  fid_t fid;
 
   switch( CTX_CNTRL )
   { case FRG_FIRST_CALL:
@@ -1753,7 +1772,13 @@ trie_gen(term_t Trie, term_t Key, term_t Value,
     }
     case FRG_REDO:
       state = CTX_PTR;
-      break;
+      if ( gen_state_leaf(state)->value ||
+	   next_choice(state PASS_LD) )		/* pending choice was deleted */
+      { break;
+      } else
+      { clear_trie_state(state);
+	return FALSE;
+      }
     case FRG_CUTTED:
       state = CTX_PTR;
       clear_trie_state(state);
@@ -1763,15 +1788,17 @@ trie_gen(term_t Trie, term_t Key, term_t Value,
       return FALSE;
   }
 
-  fid = PL_open_foreign_frame();
+  Mark(fli_context->mark);
   for( ; !isEmptyBuffer(&state->choicepoints); next_choice(state PASS_LD) )
-  { int rc;
+  { for(;;)
+    { int rc;
+      size_t asize = aTop - aBase; /* using the argument stack may be dubious */
 
-    for(;;)
-    { if ( (rc=unify_trie_path(Key, &n, state PASS_LD)) == TRUE )
+      if ( (rc=unify_trie_path(Key, &n, state PASS_LD)) == TRUE )
 	break;
 
-      PL_rewind_foreign_frame(fid);
+      aTop = aBase+asize;
+      Undo(fli_context->mark);
       if ( rc == FALSE )
 	goto next;
 
@@ -1791,7 +1818,7 @@ trie_gen(term_t Trie, term_t Key, term_t Value,
       { if ( !state->allocated )
 	{ trie_gen_state *nstate = allocForeignState(sizeof(*state));
 	  TmpBuffer nchp = &nstate->choicepoints;
-	  TmpBuffer ochp =  &state->choicepoints;
+	  TmpBuffer ochp = &state->choicepoints;
 
 	  nstate->trie = state->trie;
 	  nstate->allocated = TRUE;
@@ -1808,23 +1835,20 @@ trie_gen(term_t Trie, term_t Key, term_t Value,
 
 	  state = nstate;
 	}
-	PL_close_foreign_frame(fid);
 	ForeignRedoPtr(state);
       } else
       { clear_trie_state(state);
-	PL_close_foreign_frame(fid);
 	return TRUE;
       }
     } else
     { TRIE_STAT_INC(state->trie, gen_fail);
-      PL_rewind_foreign_frame(fid);
+      Undo(fli_context->mark);
     }
 
 next:;
   }
 
   clear_trie_state(state);
-  PL_close_foreign_frame(fid);
   return FALSE;
 }
 
