@@ -3,7 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2017, VU University Amsterdam
+    Copyright (c)  2017-2019, VU University Amsterdam
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -77,7 +78,11 @@ typedef struct trie_node
   struct trie_node *parent;
   trie_children children;
   struct
-  { struct delay_info *delayinfo;
+  { struct delay_info *delayinfo;	/* can be unified with children */
+    struct
+    { unsigned deleted : 1;		/* Deleted for incremental re-evaluation */
+      unsigned unconditional : 1;	/* Previous answer was unconditional */
+    } idg;
   } data;
 } trie_node;
 
@@ -108,7 +113,7 @@ typedef struct trie
   struct
   { struct worklist *worklist;		/* tabling worklist */
     trie_node	    *variant;		/* node in variant trie */
-    fastheap_term   *skeleton;		/* Wrapper-Vars */
+    struct idg_node *IDG;		/* Node in the IDG graph */
   } data;
 } trie;
 
@@ -123,12 +128,13 @@ COMMON(void)	trie_empty(trie *trie);
 COMMON(void)	trie_clean(trie *trie);
 COMMON(void)	trie_delete(trie *trie, trie_node *node, int prune);
 COMMON(void)	prune_node(trie *trie, trie_node *n);
-COMMON(trie *)	get_trie_form_node(trie_node *node);
+COMMON(trie *)	get_trie_from_node(trie_node *node);
 COMMON(int)	is_ground_trie_node(trie_node *node);
 COMMON(int)	get_trie(term_t t, trie **tp);
+COMMON(int)	get_trie_noex(term_t t, trie **tp);
 COMMON(int)	unify_trie_term(trie_node *node, term_t term ARG_LD);
 COMMON(int)	trie_lookup(trie *trie, trie_node **nodep, Word k,
-			    int add ARG_LD);
+			    int add, TmpBuffer vars ARG_LD);
 COMMON(int)	trie_error(int rc, term_t culprit);
 COMMON(atom_t)	trie_symbol(trie *trie);
 COMMON(trie *)	symbol_trie(atom_t symbol);
